package csci7818.xml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/*
 * Lots of info/examples at http://www.jdom.org/downloads/docs.html
 */
public class JdomExample {

	public static void main(String[] args) throws JDOMException, IOException {
		JdomExample example = new JdomExample();
		example.readXml("books.xml");
		example.writeXml("classes.xml");
	}
	
	/**
	 * Reads 2 child deep xml file and write its contents to the console; the obvious
	 * improvement would be to add recursion so it can process an N deep tree
	 */
	@SuppressWarnings("unchecked")
	public void readXml(String filename) throws JDOMException, IOException {
		/*
		 * Construct a SAX (really xerces) parser for the xml file
		 */
		SAXBuilder builder = new SAXBuilder();
		/*
		 * Parse it to build the JDOM Document
		 */
		Document doc = builder.build(new File(filename));
		/*
		 * Get the root level element of the xml file
		 */
		Element root = doc.getRootElement();
		System.out.println("Root: "+root.getName());
		
		List<Element> children = root.getChildren();
	
		for (Element child : children) {
			System.out.println("\tNode: "+child.getName());
			System.out.println("\tAttributes: "+child.getAttributes());
			List<Element> grandChildren = child.getChildren();
			for (Element grandchild : grandChildren) {
				System.out.println("\t\tNode: "+grandchild.getName());
				System.out.println("\t\tAttributes: "+grandchild.getAttributes());
				System.out.println("\t\tValue: "+grandchild.getText());
			}
		}
	}
	
	/**
	 * Write an xml document to disk from an in memory generated xml tree
	 */
	public void writeXml(String filename) throws IOException {
		/*
		 * Generate a namespace to use with all the elements
		 */
		Namespace namespace = Namespace.getNamespace("classxml", "http://www.class.com");
		/*
		 * Create a root element
		 */
		Element root = new Element("classes",namespace);
		
		/*
		 * Sample data to write to XML
		 */
		String[] classNames = {"CS100", "CS201", "CS302", "CS403"};
		String[] classDesc = {"Intro to Programming", "Intro to Databases", "Algorithms", "Data Structures"};
		
		/*
		 * Iterate over the sample data and generate an element for each, with a description attribute
		 * and name for the textual value
		 */
		for (int i = 0; i < 4; i++) {
			Element aclass = new Element("class",namespace);
			aclass.setText(classNames[i]);
			aclass.setAttribute("desc", classDesc[i]);
			root.addContent(aclass);
		}
		
		/*
		 * Put the root element in an xml document
		 */
		Document doc = new Document(root);
		
		/*
		 * Instantiate a file object and output stream to it, then pass is to the JDOM XML Outputter to write to disk
		 */
		File xml = new File(filename);
		// delete it if it already exists
		if (xml.exists()) {
			xml.delete();
		}
		FileOutputStream fos = new FileOutputStream(xml);
		XMLOutputter outputter = new XMLOutputter();
		// This writes the XML out with line breaks and indents
		outputter.setFormat(Format.getPrettyFormat());
		outputter.output(doc, fos);

	}

}
